//applewindow.cpp

/*---------------------------------------------------------
	Author: Benjamin Kenwright (bkenwright@xbdev.com)
	File: applewindow.cpp
	Details: The program, creates a fully functional
	  desktop window, that looks like a big red apple.
----------------------------------------------------------*/


#include <windows.h>
//#include "memory_leaks.h"//need to be put after <windows.h>

#include "resource.h"

//MACROS
#define _T		TEXT
#define BITMAP_WIDTH	200
#define BITMAP_HEIGHT	200


//FUNCTION DEFINITIONS
HRGN CreateRgnFromBitmap(HWND, HBITMAP*, int, int, COLORREF);

LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM);

int WINAPI WinMain (HINSTANCE hInst, HINSTANCE hPrevInst,
					PSTR szCmdLine, int iCmdShow)
{
	MSG					msg;
	WNDCLASS			wndclass;

	wndclass.style			= CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc	= WndProc;
	wndclass.cbClsExtra		= 0;
	wndclass.cbWndExtra		= 0;
	wndclass.hInstance		= hInst;
	wndclass.hIcon			= (HICON)LoadIcon(NULL,IDI_APPLICATION);
	wndclass.hCursor		= (HCURSOR)LoadCursor(NULL, IDC_ARROW);
	wndclass.hbrBackground	= (HBRUSH)GetStockObject(WHITE_BRUSH);
	wndclass.lpszMenuName	= 0;
	wndclass.lpszClassName	= _T("Apple Window");

	if(!RegisterClass(&wndclass))  
	{	
		MessageBox(NULL,_T("Error Registering Window"),_T("Apple Window"), MB_ICONERROR);
		return 0;
	}

	HWND hwnd = CreateWindow (_T("Apple Window"), _T("Apple Window - Red Apple"),
		WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT, CW_USEDEFAULT,
		200, 200,
		NULL, NULL, hInst, NULL);
	
	ShowWindow(hwnd, iCmdShow);
	UpdateWindow(hwnd);

	while (GetMessage(&msg,NULL,0,0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return (int)msg.wParam;
}
HINSTANCE hInstance;
LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{	
	
	switch(message)
	{
	case WM_CREATE:
		hInstance = ((LPCREATESTRUCT)lParam)->hInstance;
		{
		//Load the bitmap
		HBITMAP hbit = LoadBitmap(hInstance,
							MAKEINTRESOURCE(IDB_BITMAP));
		//Create the region from the bitmap and size the window to it
		HRGN hrgn = CreateRgnFromBitmap(hwnd,&hbit, 
									BITMAP_WIDTH,BITMAP_HEIGHT, 
									RGB(0,0,255));
		SetWindowRgn(hwnd,hrgn,FALSE);
		//Resize the window to the size of the bitmap
		RECT* lpRect = new RECT;
		GetWindowRect(hwnd,lpRect);
		MoveWindow(hwnd,lpRect->left,lpRect->top,BITMAP_WIDTH,BITMAP_HEIGHT,TRUE);
		delete lpRect;
		//Tidy up
		DeleteObject(hrgn);
		DeleteObject(hbit);
		}
		break;
	case WM_PAINT:
		{
		HDC hdcClient = GetWindowDC(hwnd);
		HDC hdcMem = CreateCompatibleDC(hdcClient);
		
		HBITMAP hbit = LoadBitmap(hInstance,MAKEINTRESOURCE(IDB_BITMAP));
		SelectObject(hdcMem,hbit);

		BitBlt(hdcClient,0,0,BITMAP_WIDTH,BITMAP_HEIGHT,
						hdcMem,0,0,SRCCOPY);
		
		ReleaseDC(hwnd,hdcClient);
		ReleaseDC(hwnd,hdcMem);
		DeleteObject(hbit);
		}
		
		ValidateRect(hwnd,NULL);
		break;

	case WM_SIZE:
		break;
	case WM_DESTROY:
		//DumpUnfreed();
		PostQuitMessage(0);
		return 0;
	}
	return DefWindowProc(hwnd, message, wParam, lParam);
}

/*-----------Creates a region from a bitmap----------------*/

HRGN CreateRgnFromBitmap(HWND hwnd, 
						 HBITMAP* hBitmap, int w, int h, 
						 COLORREF color)
{
	HRGN hrgn = CreateRectRgn(0,0,w,h);
	
	HDC hdcScreen = GetDC(hwnd);
	HDC hdc = CreateCompatibleDC(hdcScreen);
	ReleaseDC(hwnd,hdcScreen);
	SelectObject(hdc,*hBitmap);

	for(int height=0; height<h; height++)
	{
		for (int width=0; width<w; width++)
		{
			COLORREF cr = GetPixel(hdc,width,height);
			if (cr == color)
			{
				HRGN hrgntemp = CreateRectRgn(width,height,width+1,height+1);
				CombineRgn(hrgn, hrgn, hrgntemp, RGN_XOR);
			
				DeleteObject(hrgntemp);
			}
		}
	}
	
	ReleaseDC(hwnd,hdc);	
	return (hrgn);
}





